@@ -9,6 +9,7 @@
 //
 
 use std::path::PathBuf;
+use std::str::FromStr;
 
 use clap::crate_authors;
 use clap::crate_version;
@@ -26,6 +27,12 @@ pub fn cli<'a>() -> App<'a> {
         .version(crate_version!())
         .about("Generic Build Orchestration System for building linux packages with docker")
 
+        .after_help(r#"
+            The following environment variables can be passed to butido:
+
+                RUST_LOG - to enable logging, for exact usage see the rust cookbook
+        "#)
+
         .arg(Arg::new("hide_bars")
             .required(false)
             .multiple(false)
@@ -38,21 +45,33 @@ pub fn cli<'a>() -> App<'a> {
             .multiple(false)
             .long("db-url")
             .value_name("HOST")
-            .about("Overwrite the database host set via configuration. Can also be overriden via environment, but this setting has precedence.")
+            .about("Override the database host")
+            .long_about(indoc::indoc!(r#"
+                Override the database host set via configuration.
+                Can also be overriden via environment variable 'BUTIDO_DATABASE_HOST', but this setting has precedence.
+            "#))
         )
         .arg(Arg::new("database_port")
             .required(false)
             .multiple(false)
             .long("db-port")
             .value_name("PORT")
-            .about("Overwrite the database port set via configuration. Can also be overriden via environment, but this setting has precedence.")
+            .about("Override the database port")
+            .long_about(indoc::indoc!(r#"
+                Override the database port set via configuration.
+                Can also be overriden via environment 'BUTIDO_DATABASE_PORT', but this setting has precedence.
+            "#))
         )
         .arg(Arg::new("database_user")
             .required(false)
             .multiple(false)
             .long("db-user")
             .value_name("USER")
-            .about("Overwrite the database user set via configuration. Can also be overriden via environment, but this setting has precedence.")
+            .about("Override the database user")
+            .long_about(indoc::indoc!(r#"
+                Override the database user set via configuration.
+                Can also be overriden via environment 'BUTIDO_DATABASE_USER', but this setting has precedence.
+            "#))
         )
         .arg(Arg::new("database_password")
             .required(false)
@@ -60,14 +79,33 @@ pub fn cli<'a>() -> App<'a> {
             .long("db-password")
             .alias("db-pw")
             .value_name("PASSWORD")
-            .about("Overwrite the database password set via configuration. Can also be overriden via environment, but this setting has precedence.")
+            .about("Override the database password")
+            .long_about(indoc::indoc!(r#"
+                Override the database password set via configuration.
+                Can also be overriden via environment 'BUTIDO_DATABASE_PASSWORD', but this setting has precedence.
+            "#))
         )
         .arg(Arg::new("database_name")
             .required(false)
             .multiple(false)
             .long("db-name")
             .value_name("NAME")
-            .about("Overwrite the database name set via configuration. Can also be overriden via environment, but this setting has precedence.")
+            .about("Override the database name")
+            .long_about(indoc::indoc!(r#"
+                Override the database name set via configuration.
+                Can also be overriden via environment 'BUTIDO_DATABASE_NAME', but this setting has precedence.
+            "#))
+        )
+        .arg(Arg::new("database_connection_timeout")
+            .required(false)
+            .multiple(false)
+            .long("db-timeout")
+            .value_name("TIMEOUT")
+            .about("Override the database connection timeout")
+            .long_about(indoc::indoc!(r#"
+                Override the database connection timeout set via configuration.
+                Can also be overriden via environment 'BUTIDO_DATABASE_CONNECTION_TIMEOUT', but this setting has precedence.
+            "#))
         )
 
         .subcommand(App::new("generate-completions")
@@ -89,8 +127,8 @@ pub fn cli<'a>() -> App<'a> {
                 .version(crate_version!())
                 .about("Start a database CLI, if installed on the current host")
                 .long_about(indoc::indoc!(r#"
-                    Starts a database shell on the configured database using one of the following
-                    programs:
+                    Starts a database shell on the configured database using one of the following programs:
+
                         - psql
                         - pgcli
 
@@ -107,6 +145,14 @@ pub fn cli<'a>() -> App<'a> {
                 )
             )
 
+            .subcommand(App::new("setup")
+                .version(crate_version!())
+                .about("Run the database setup")
+                .long_about(indoc::indoc!(r#"
+                    Run the database setup migrations
+                "#))
+            )
+
             .subcommand(App::new("artifacts")
                 .version(crate_version!())
                 .about("List artifacts from the DB")
@@ -152,6 +198,19 @@ pub fn cli<'a>() -> App<'a> {
                 )
             )
 
+            .subcommand(App::new("submit")
+                .version(crate_version!())
+                .about("Show details about one specific submit")
+                .arg(Arg::new("submit")
+                    .required(true)
+                    .multiple(false)
+                    .index(1)
+                    .takes_value(true)
+                    .value_name("SUBMIT")
+                    .about("The Submit to show details about")
+                )
+            )
+
             .subcommand(App::new("submits")
                 .version(crate_version!())
                 .about("List submits from the DB")
@@ -180,6 +239,14 @@ pub fn cli<'a>() -> App<'a> {
                     .about("Only list submits that had the root package PKG")
                     .conflicts_with("with_pkg")
                 )
+                .arg(Arg::new("limit")
+                    .required(false)
+                    .multiple(false)
+                    .long("limit")
+                    .takes_value(true)
+                    .value_name("LIMIT")
+                    .about("Only list LIMIT submits")
+                )
             )
 
             .subcommand(App::new("jobs")
@@ -212,6 +279,40 @@ pub fn cli<'a>() -> App<'a> {
                     .value_name("KV")
                     .about("Filter for this \"key=value\" environment variable")
                 )
+
+                .arg(Arg::new("limit")
+                    .required(false)
+                    .multiple(false)
+                    .long("limit")
+                    .short('L')
+                    .takes_value(true)
+                    .value_name("LIMIT")
+                    .about("Only list newest LIMIT jobs instead of all")
+                )
+
+                .arg(arg_older_than_date("List only jobs older than DATE"))
+                .arg(arg_newer_than_date("List only jobs newer than DATE"))
+
+                .arg(Arg::new("endpoint")
+                    .required(false)
+                    .multiple(false)
+                    .long("endpoint")
+                    .short('e')
+                    .takes_value(true)
+                    .value_name("ENDPOINT")
+                    .about("Only show jobs from ENDPOINT")
+                )
+
+                .arg(Arg::new("package")
+                    .required(false)
+                    .multiple(false)
+                    .long("package")
+                    .short('p')
+                    .takes_value(true)
+                    .value_name("PKG")
+                    .about("Only show jobs for PKG")
+                )
+
             )
 
             .subcommand(App::new("job")
@@ -263,6 +364,18 @@ pub fn cli<'a>() -> App<'a> {
                 .arg(script_arg_highlight())
                 .arg(script_arg_no_highlight())
             )
+            .subcommand(App::new("log-of")
+                .version(crate_version!())
+                .about("Print log of a job, short version of 'db job --log'")
+                .arg(Arg::new("job_uuid")
+                    .required(true)
+                    .multiple(false)
+                    .index(1)
+                    .takes_value(true)
+                    .value_name("UUID")
+                    .about("The id of the Job")
+                )
+            )
             .subcommand(App::new("releases")
                 .version(crate_version!())
                 .about("List releases")
@@ -273,6 +386,28 @@ pub fn cli<'a>() -> App<'a> {
                     .takes_value(false)
                     .about("Format output as CSV")
                 )
+
+                .arg(arg_older_than_date("List only releases older than DATE"))
+                .arg(arg_newer_than_date("List only releases newer than DATE"))
+
+                .arg(Arg::new("store")
+                    .required(false)
+                    .multiple(false)
+                    .long("to")
+                    .takes_value(true)
+                    .value_name("STORE")
+                    .about("List only releases to STORE")
+                )
+
+                .arg(Arg::new("package")
+                    .required(false)
+                    .multiple(false)
+                    .long("package")
+                    .short('p')
+                    .takes_value(true)
+                    .value_name("PKG")
+                    .about("Only list releases for package PKG")
+                )
             )
         )
 
@@ -299,14 +434,20 @@ pub fn cli<'a>() -> App<'a> {
                 .multiple(false)
                 .takes_value(false)
                 .long("no-verify")
-                .about("Do not perform a hash sum check on all packages in the dependency tree before starting the build")
+                .about("Skip hashsum check")
+                .long_about(indoc::indoc!(r#"
+                    Do not perform a hash sum check on all packages in the dependency tree before starting the build.
+                "#))
             )
             .arg(Arg::new("no_lint")
                 .required(false)
                 .multiple(false)
                 .takes_value(false)
                 .long("no-lint")
-                .about("Do not perform script linting before starting the build")
+                .about("Skip linting")
+                .long_about(indoc::indoc!(r#"
+                    Do not perform script linting before starting the build.
+                "#))
             )
 
             .arg(Arg::new("staging_dir")
@@ -334,7 +475,11 @@ pub fn cli<'a>() -> App<'a> {
                 .short('E')
                 .long("env")
                 .validator(env_pass_validator)
-                .about("Pass these variables to each build job (expects \"key=value\" or name of variable available in ENV)")
+                .about("Pass environment variable to all build jobs")
+                .long_about(indoc::indoc!(r#"
+                    Pass these variables to each build job.
+                    This argument expects \"key=value\" or name of variable available in ENV
+                "#))
             )
 
             .arg(Arg::new("image")
@@ -352,7 +497,13 @@ pub fn cli<'a>() -> App<'a> {
                 .multiple(false)
                 .long("write-log")
                 .short('L')
-                .about("Write the log not only to database, but also in a plain-text-file")
+                .about("Write log to disk as well")
+                .long_about(indoc::indoc!(r#"
+                    With this flag set, butido does not only write the build logs to database, but also to the configured
+                    log directory.
+
+                    The log of a build is written to `<log_dir>/<build id>.log`.
+                "#))
             )
         )
 
@@ -691,6 +842,24 @@ pub fn cli<'a>() -> App<'a> {
                     .about("Overwrite existing cache entry")
                 )
             )
+            .subcommand(App::new("of")
+                .version(crate_version!())
+                .about("Get the pathes of the sources of a package")
+                .arg(Arg::new("package_name")
+                    .required(false)
+                    .multiple(false)
+                    .index(1)
+                    .value_name("PKG")
+                    .about("Get the source file pathes for this package")
+                )
+                .arg(Arg::new("package_version")
+                    .required(false)
+                    .multiple(false)
+                    .index(2)
+                    .value_name("VERSION")
+                    .about("Get the source file pathes for the package in this version")
+                )
+            )
         )
 
         .subcommand(App::new("release")
@@ -792,6 +961,13 @@ pub fn cli<'a>() -> App<'a> {
                     .about("Dont be interactive (only with --update at the moment)")
                     .requires("package_do_update")
                 )
+                .arg(Arg::new("quiet")
+                    .required(false)
+                    .multiple(false)
+                    .long("quiet")
+                    .short('q')
+                    .about("Don't print pathes to released filesfiles  after releases are complete")
+                )
             )
 
         )
@@ -888,26 +1064,23 @@ pub fn cli<'a>() -> App<'a> {
                 .subcommand(App::new("prune")
                     .version(crate_version!())
                     .about("Remove exited containers")
-                    .arg(Arg::new("older_than")
+                    .arg(arg_older_than_date("Prune only containers older than DATE"))
+                    .arg(arg_newer_than_date("Prune only containers newer than DATE"))
+                )
+                .subcommand(App::new("stop")
+                    .version(crate_version!())
+                    .about("Stop running containers")
+                    .arg(arg_older_than_date("Stop only containers older than DATE"))
+                    .arg(arg_newer_than_date("Stop only containers newer than DATE"))
+                    .arg(Arg::new("timeout")
                         .required(false)
                         .multiple(false)
-                        .long("older-than")
+                        .long("timeout")
+                        .short('t')
                         .takes_value(true)
-                        .value_name("DATE")
-                        .about("List only containers that are older than DATE")
-                        .validator(parse_date_from_string)
-                        .conflicts_with("newer_than")
-                    )
-
-                    .arg(Arg::new("newer_than")
-                        .required(false)
-                        .multiple(false)
-                        .long("newer-than")
-                        .takes_value(true)
-                        .value_name("DATE")
-                        .about("List only containers that are newer than DATE")
-                        .validator(parse_date_from_string)
-                        .conflicts_with("older_than")
+                        .value_name("TIMEOUT")
+                        .about("Timeout in seconds")
+                        .validator(parse_u64)
                     )
                 )
                 .subcommand(App::new("list")
@@ -938,26 +1111,27 @@ pub fn cli<'a>() -> App<'a> {
                         .about("List only containers of IMAGE")
                     )
 
-                    .arg(Arg::new("older_than")
+                    .arg(arg_older_than_date("List only containers older than DATE"))
+                    .arg(arg_newer_than_date("List only containers newer than DATE"))
+                )
+                .subcommand(App::new("top")
+                    .version(crate_version!())
+                    .about("List the processes of all containers")
+                    .arg(Arg::new("csv")
                         .required(false)
                         .multiple(false)
-                        .long("older-than")
-                        .takes_value(true)
-                        .value_name("DATE")
-                        .about("List only containers that are older than DATE")
-                        .validator(parse_date_from_string)
-                        .conflicts_with("newer_than")
+                        .long("csv")
+                        .takes_value(false)
+                        .about("List top output as CSV")
                     )
-
-                    .arg(Arg::new("newer_than")
+                    .arg(Arg::new("limit")
                         .required(false)
                         .multiple(false)
-                        .long("newer-than")
+                        .long("limit")
                         .takes_value(true)
-                        .value_name("DATE")
-                        .about("List only containers that are newer than DATE")
-                        .validator(parse_date_from_string)
-                        .conflicts_with("older_than")
+                        .value_name("LIMIT")
+                        .about("Only list LIMIT processes for each container")
+                        .validator(parse_usize)
                     )
                 )
             )
@@ -1118,8 +1292,89 @@ fn dir_exists_validator(s: &str) -> Result<(), String> {
     }
 }
 
+fn arg_older_than_date(about: &str) -> Arg<'_> {
+    Arg::new("older_than")
+        .required(false)
+        .multiple(false)
+        .long("older-than")
+        .takes_value(true)
+        .value_name("DATE")
+        .about(about)
+        .long_about(r#"
+            DATE can be a freeform date, for example '2h'
+            It can also be a exact date: '2020-01-01 00:12:45'
+            If the hour-minute-second part is omitted, " 00:00:00" is appended automatically.
+
+            Supported suffixes:
+
+                nsec, ns -- nanoseconds
+                usec, us -- microseconds
+                msec, ms -- milliseconds
+                seconds, second, sec, s
+                minutes, minute, min, m
+                hours, hour, hr, h
+                days, day, d
+                weeks, week, w
+                months, month, M -- defined as 30.44 days
+                years, year, y -- defined as 365.25 days
+
+        "#)
+        .validator(parse_date_from_string)
+}
+
+fn arg_newer_than_date(about: &str) -> Arg<'_> {
+    Arg::new("newer_than")
+        .required(false)
+        .multiple(false)
+        .long("newer-than")
+        .takes_value(true)
+        .value_name("DATE")
+        .about(about)
+        .long_about(r#"
+            DATE can be a freeform date, for example '2h'
+            It can also be a exact date: '2020-01-01 00:12:45'
+            If the hour-minute-second part is omitted, " 00:00:00" is appended automatically.
+
+            Supported suffixes:
+
+                nsec, ns -- nanoseconds
+                usec, us -- microseconds
+                msec, ms -- milliseconds
+                seconds, second, sec, s
+                minutes, minute, min, m
+                hours, hour, hr, h
+                days, day, d
+                weeks, week, w
+                months, month, M -- defined as 30.44 days
+                years, year, y -- defined as 365.25 days
+
+        "#)
+        .validator(parse_date_from_string)
+}
+
 fn parse_date_from_string(s: &str) -> std::result::Result<(), String> {
-    humantime::parse_rfc3339_weak(s).map_err(|e| e.to_string()).map(|_| ())
+    humantime::parse_duration(s)
+        .map_err(|e| e.to_string())
+        .map(|_| ())
+        .or_else(|_| {
+            humantime::parse_rfc3339_weak(s)
+                .map_err(|e| e.to_string())
+                .map(|_| ())
+        })
+        .or_else(|_| {
+            let s = format!("{} 00:00:00", s);
+            humantime::parse_rfc3339_weak(&s)
+                .map_err(|e| e.to_string())
+                .map(|_| ())
+        })
+}
+
+fn parse_usize(s: &str) -> std::result::Result<(), String> {
+    usize::from_str(s) .map_err(|e| e.to_string()).map(|_| ())
+}
+
+fn parse_u64(s: &str) -> std::result::Result<(), String> {
+    u64::from_str(s).map_err(|e| e.to_string()).map(|_| ())
 }
 
 #[cfg(test)]
