@@ -8,9 +8,11 @@
 // SPDX-License-Identifier: EPL-2.0
 //
 
+//! Implementation of the 'source' subcommand
+
 use std::io::Write;
 use std::path::PathBuf;
-use std::sync::Arc;
+use std::convert::TryFrom;
 
 use anyhow::anyhow;
 use anyhow::Context;
@@ -42,8 +44,9 @@ pub async fn source(
         Some(("list-missing", matches)) => list_missing(matches, config, repo).await,
         Some(("url", matches)) => url(matches, repo).await,
         Some(("download", matches)) => download(matches, config, repo, progressbars).await,
+        Some(("of", matches)) => of(matches, config, repo).await,
         Some((other, _)) => return Err(anyhow!("Unknown subcommand: {}", other)),
-        None => return Err(anyhow!("No subcommand")),
+        None => Err(anyhow!("No subcommand")),
     }
 }
 
@@ -60,8 +63,7 @@ pub async fn verify(
         .map(PackageName::from);
     let pvers = matches
         .value_of("package_version")
-        .map(String::from)
-        .map(PackageVersionConstraint::new)
+        .map(PackageVersionConstraint::try_from)
         .transpose()?;
 
     let packages = repo
@@ -86,21 +88,19 @@ pub(in crate::commands) async fn verify_impl<'a, I>(
 where
     I: Iterator<Item = &'a Package> + 'a,
 {
-    let multi = Arc::new({
-        let mp = indicatif::MultiProgress::new();
-        if progressbars.hide() {
-            mp.set_draw_target(indicatif::ProgressDrawTarget::hidden());
-        }
-        mp
-    });
-
-    let results = packages
+    let sources = packages
         .map(|p| sc.sources_for(p).into_iter())
         .flatten()
-        .map(|src| (multi.clone(), src))
-        .map(|(multi, source)| async move {
+        .collect::<Vec<_>>();
+
+    let bar = progressbars.bar();
+    bar.set_message("Verifying sources");
+    bar.set_length(sources.len() as u64);
+
+    let results = sources.into_iter()
+        .map(|src| (bar.clone(), src))
+        .map(|(bar, source)| async move {
             trace!("Verifying: {}", source.path().display());
-            let bar = multi.add(progressbars.bar());
             if source.path().exists() {
                 trace!("Exists: {}", source.path().display());
                 source.verify_hash().await.with_context(|| {
@@ -108,37 +108,36 @@ where
                 })?;
 
                 trace!("Success verifying: {}", source.path().display());
-                let msg = format!("Ok: {}", source.path().display()).green();
-                bar.finish_with_message(&msg);
-                Ok(msg)
+                bar.inc(1);
+                Ok(())
             } else {
                 trace!("Failed verifying: {}", source.path().display());
-                bar.finish_with_message(&"Error".red());
+                bar.inc(1);
                 Err(anyhow!("Source missing: {}", source.path().display()))
             }
         })
         .collect::<futures::stream::FuturesUnordered<_>>()
-        .collect::<Vec<Result<_>>>();
+        .collect::<Vec<Result<_>>>()
+        .await;
 
-    let multibar_block = tokio::task::spawn_blocking(move || multi.join());
-    let (results, _) = tokio::join!(results, multibar_block);
     info!("Verification processes finished");
 
+    if results.iter().any(Result::is_err) {
+        bar.finish_with_message("Source verification failed");
+    } else {
+        bar.finish_with_message("Source verification successfull");
+    }
+
     let out = std::io::stdout();
     let mut any_error = false;
     for result in results {
-        match result {
-            Err(e) => {
-                let mut outlock = out.lock();
-                any_error = true;
-                for cause in e.chain() {
-                    let _ = writeln!(outlock, "Error: {}", cause.to_string().red());
-                }
-                let _ = writeln!(outlock);
-            }
-            Ok(s) => {
-                let _ = writeln!(out.lock(), "{}", s.green());
+        if let Err(e) = result {
+            let mut outlock = out.lock();
+            any_error = true;
+            for cause in e.chain() {
+                let _ = writeln!(outlock, "Error: {}", cause.to_string().red());
             }
+            let _ = writeln!(outlock);
         }
     }
 
@@ -158,7 +157,7 @@ pub async fn list_missing(_: &ArgMatches, config: &Configuration, repo: Reposito
 
     repo.packages().try_for_each(|p| {
         for source in sc.sources_for(p) {
-            if !source.exists() {
+            if !source.path().exists() {
                 writeln!(
                     outlock,
                     "{} {} -> {}",
@@ -183,8 +182,7 @@ pub async fn url(matches: &ArgMatches, repo: Repository) -> Result<()> {
         .map(PackageName::from);
     let pvers = matches
         .value_of("package_version")
-        .map(String::from)
-        .map(PackageVersionConstraint::new)
+        .map(PackageVersionConstraint::try_from)
         .transpose()?;
 
     repo.packages()
@@ -225,8 +223,7 @@ pub async fn download(
         .map(PackageName::from);
     let pvers = matches
         .value_of("package_version")
-        .map(String::from)
-        .map(PackageVersionConstraint::new)
+        .map(PackageVersionConstraint::try_from)
         .transpose()?;
     let multi = {
         let mp = indicatif::MultiProgress::new();
@@ -248,9 +245,10 @@ pub async fn download(
         .map(|p| {
             sc.sources_for(p).into_iter().map(|source| {
                 let bar = multi.add(progressbars.spinner());
-                bar.set_message(&format!("Downloading {}", source.url()));
+                bar.set_message(format!("Downloading {}", source.url()));
                 async move {
-                    if !source.exists() && source.download_manually() {
+                    let source_path_exists = source.path().exists();
+                    if !source_path_exists && source.download_manually() {
                         return Err(anyhow!(
                             "Cannot download source that is marked for manual download"
                         ))
@@ -259,47 +257,67 @@ pub async fn download(
                         .map_err(Error::from);
                     }
 
-                    if source.exists() && !force {
+                    if source_path_exists && !force {
                         Err(anyhow!("Source exists: {}", source.path().display()))
                     } else {
-                        if source.exists() {
-                            let _ = source.remove_file().await?;
-                        }
+                        async fn perform_download(source: &SourceEntry, bar: &indicatif::ProgressBar) -> Result<()> {
+                            trace!("Creating: {:?}", source);
+                            let file = source.create().await.with_context(|| {
+                                anyhow!(
+                                    "Creating source file destination: {}",
+                                    source.path().display()
+                                )
+                            })?;
 
-                        trace!("Creating: {:?}", source);
-                        let file = source.create().await.with_context(|| {
-                            anyhow!(
-                                "Creating source file destination: {}",
-                                source.path().display()
-                            )
-                        })?;
+                            let mut file = tokio::io::BufWriter::new(file);
+                            let response = match reqwest::get(source.url().as_ref()).await {
+                                Ok(resp) => resp,
+                                Err(e) => {
+                                    bar.finish_with_message(format!("Failed: {}", source.url()));
+                                    return Err(e).with_context(|| anyhow!("Downloading '{}'", source.url()))
+                                }
+                            };
 
-                        let mut file = tokio::io::BufWriter::new(file);
-                        let response = reqwest::get(source.url().as_ref())
-                            .await
-                            .with_context(|| anyhow!("Downloading '{}'", source.url()))?;
-
-                        if let Some(len) = response.content_length() {
-                            bar.set_length(len);
-                        }
-                        let mut stream = reqwest::get(source.url().as_ref()).await?.bytes_stream();
-                        let mut bytes_written = 0;
-                        while let Some(bytes) = stream.next().await {
-                            let bytes = bytes?;
-                            file.write_all(bytes.as_ref()).await?;
-                            bytes_written += bytes.len();
-
-                            bar.inc(bytes.len() as u64);
                             if let Some(len) = response.content_length() {
-                                bar.set_message(&format!("Downloading {} ({}/{} bytes)", source.url(), bytes_written, len));
-                            } else {
-                                bar.set_message(&format!("Downloading {} ({} bytes)", source.url(), bytes_written));
+                                bar.set_length(len);
+                            }
+
+                            let mut stream = reqwest::get(source.url().as_ref()).await?.bytes_stream();
+                            let mut bytes_written = 0;
+                            while let Some(bytes) = stream.next().await {
+                                let bytes = bytes?;
+                                file.write_all(bytes.as_ref()).await?;
+                                bytes_written += bytes.len();
+
+                                bar.inc(bytes.len() as u64);
+                                if let Some(len) = response.content_length() {
+                                    bar.set_message(format!("Downloading {} ({}/{} bytes)", source.url(), bytes_written, len));
+                                } else {
+                                    bar.set_message(format!("Downloading {} ({} bytes)", source.url(), bytes_written));
+                                }
+                            }
+
+                            file.flush()
+                                .await
+                                .map_err(Error::from)
+                                .map(|_| ())
+                        }
+
+                        if source_path_exists /* && force is implied by 'if' above*/ {
+                            if let Err(e) = source.remove_file().await {
+                                bar.finish_with_message(format!("Failed to remove existing file: {}", source.path().display()));
+                                return Err(e)
                             }
                         }
 
-                        file.flush().await?;
-                        bar.finish_with_message("Finished download");
-                        Ok(())
+
+                        if let Err(e) = perform_download(&source, &bar).await {
+                            bar.finish_with_message(format!("Failed: {}", source.url()));
+                            Err(e)
+                        } else {
+                            bar.finish_with_message(format!("Finished: {}", source.url()));
+                            Ok(())
+                        }
                     }
                 }
             })
@@ -312,3 +330,48 @@ pub async fn download(
     let (r, _) = tokio::join!(r, multibar_block);
     r.into_iter().collect()
 }
+
+async fn of(
+    matches: &ArgMatches,
+    config: &Configuration,
+    repo: Repository,
+) -> Result<()> {
+    let cache = PathBuf::from(config.source_cache_root());
+    let sc = SourceCache::new(cache);
+    let pname = matches
+        .value_of("package_name")
+        .map(String::from)
+        .map(PackageName::from);
+    let pvers = matches
+        .value_of("package_version")
+        .map(PackageVersionConstraint::try_from)
+        .transpose()?;
+
+    repo.packages()
+        .filter(|p| pname.as_ref().map(|n| p.name() == n).unwrap_or(true))
+        .filter(|p| {
+            pvers
+                .as_ref()
+                .map(|v| v.matches(p.version()))
+                .unwrap_or(true)
+        })
+        .map(|p| {
+            let pathes = sc.sources_for(p)
+                .into_iter()
+                .map(|source| source.path())
+                .collect::<Vec<PathBuf>>();
+
+            (p, pathes)
+        })
+        .fold(Ok(std::io::stdout()), |out, (package, pathes)| {
+            out.and_then(|mut out| {
+                writeln!(out, "{} {}", package.name(), package.version())?;
+                for path in pathes {
+                    writeln!(out, "\t{}", path.display())?;
+                }
+
+                Ok(out)
+            })
+        })
+        .map(|_| ())
+}
