@@ -38,14 +38,28 @@ impl Image {
             name: image_name.as_ref(),
         };
 
-        diesel::insert_into(images::table)
-            .values(&new_image)
-            .on_conflict_do_nothing()
-            .execute(database_connection)?;
+        database_connection.transaction::<_, Error, _>(|| {
+            diesel::insert_into(images::table)
+                .values(&new_image)
+                .on_conflict_do_nothing()
+                .execute(database_connection)?;
 
-        dsl::images
-            .filter(name.eq(image_name.as_ref()))
-            .first::<Image>(database_connection)
-            .map_err(Error::from)
+            dsl::images
+                .filter(name.eq(image_name.as_ref()))
+                .first::<Image>(database_connection)
+                .map_err(Error::from)
+        })
+    }
+
+    pub fn fetch_for_job(database_connection: &PgConnection, j: &crate::db::models::Job) -> Result<Option<Image>> {
+        Self::fetch_by_id(database_connection, j.image_id)
+    }
+
+    pub fn fetch_by_id(database_connection: &PgConnection, iid: i32) -> Result<Option<Image>> {
+        match dsl::images.filter(id.eq(iid)).first::<Image>(database_connection) {
+            Err(diesel::result::Error::NotFound) => Ok(None),
+            Err(e) => Err(Error::from(e)),
+            Ok(i) => Ok(Some(i)),
+        }
     }
 }
