@@ -84,8 +84,13 @@ impl Endpoint {
             Endpoint::check_api_version_compat(epc.required_docker_api_versions().as_ref(), &ep);
         let imgs_avail = Endpoint::check_images_available(epc.required_images().as_ref(), &ep);
 
-        let (versions_compat, api_versions_compat, imgs_avail) =
-            tokio::join!(versions_compat, api_versions_compat, imgs_avail);
+        let (versions_compat, api_versions_compat, imgs_avail) = {
+            let timeout = std::time::Duration::from_secs(epc.endpoint().timeout().unwrap_or(10));
+            let versions_compat = tokio::time::timeout(timeout, versions_compat);
+            let api_versions_compat = tokio::time::timeout(timeout, api_versions_compat);
+            let imgs_avail = tokio::time::timeout(timeout, imgs_avail);
+            tokio::join!(versions_compat, api_versions_compat, imgs_avail)
+        };
 
         let _ = versions_compat.with_context(|| {
             anyhow!(
@@ -538,8 +543,8 @@ impl<'a> PreparedContainer<'a> {
             .patches()
             .iter()
             .map(|patch| async move {
-                let destination = PathBuf::from("/patches").join(patch);
-                trace!("Copying patch {} to container at /patches/{}", patch.display(), destination.display());
+                let destination = PathBuf::from(crate::consts::PATCH_DIR_PATH).join(patch);
+                trace!("Copying patch {} to container at {}/{}", crate::consts::PATCH_DIR_PATH, patch.display(), destination.display());
 
                 let mut buf = vec![];
                 tokio::fs::OpenOptions::new()
