@@ -159,6 +159,8 @@ impl JobHandle {
             .execute_script(log_sender);
 
         let logres = LogReceiver {
+            endpoint_name: endpoint_name.as_ref(),
+            container_id_chrs: container_id.chars().take(7).collect(),
             package_name: &package.name,
             package_version: &package.version,
             log_dir: self.log_dir.as_ref(),
@@ -317,6 +319,8 @@ impl JobHandle {
 }
 
 struct LogReceiver<'a> {
+    endpoint_name: &'a str,
+    container_id_chrs: String,
     package_name: &'a str,
     package_version: &'a str,
     log_dir: Option<&'a PathBuf>,
@@ -329,9 +333,35 @@ impl<'a> LogReceiver<'a> {
     async fn join(mut self) -> Result<String> {
         let mut success = None;
         let mut accu = vec![];
+
+        // Reserve a reasonable amount of elements.
+        accu.reserve(4096);
+
         let mut logfile = self.get_logfile().await.transpose()?;
 
-        while let Some(logitem) = self.log_receiver.recv().await {
+        // The timeout for the log-receive-timeout
+        //
+        // We're using a rather small timeout of just 250ms here, because we have some worktime
+        // overhead as well, and we want to ping the progressbar for the process in a way so that
+        // it updates each second.
+        // Having a timeout of 1 sec proofed to be too long to update the time field in the
+        // progress bar secondly.
+        let timeout_duration = std::time::Duration::from_millis(250);
+
+        loop {
+            // Timeout for receiving from the log receiver channel
+            // This way we can update (`tick()`) the progress bar and show the user that things are
+            // happening, even if there was no log output for several seconds.
+            let logitem = match tokio::time::timeout(timeout_duration, self.log_receiver.recv()).await {
+                Err(_ /* elapsed */) => {
+                    self.bar.tick(); // just ping the progressbar here
+                    continue
+                },
+
+                Ok(None) => break, // if the log is empty, we're done
+                Ok(Some(logitem)) => logitem,
+            };
+
             if let Some(lf) = logfile.as_mut() {
                 lf.write_all(logitem.display()?.to_string().as_bytes())
                     .await?;
@@ -348,24 +378,24 @@ impl<'a> LogReceiver<'a> {
                 }
                 LogItem::CurrentPhase(ref phasename) => {
                     trace!("Setting bar phase to {}", phasename);
-                    self.bar.set_message(&format!(
-                        "[{} {} {}]: Phase: {}",
-                        self.job_id, self.package_name, self.package_version, phasename
+                    self.bar.set_message(format!(
+                        "[{}/{} {} {} {}]: Phase: {}",
+                        self.endpoint_name, self.container_id_chrs, self.job_id, self.package_name, self.package_version, phasename
                     ));
                 }
                 LogItem::State(Ok(())) => {
                     trace!("Setting bar state to Ok");
-                    self.bar.set_message(&format!(
-                        "[{} {} {}]: State Ok",
-                        self.job_id, self.package_name, self.package_version
+                    self.bar.set_message(format!(
+                        "[{}/{} {} {} {}]: State Ok",
+                        self.endpoint_name, self.container_id_chrs, self.job_id, self.package_name, self.package_version
                     ));
                     success = Some(true);
                 }
                 LogItem::State(Err(ref e)) => {
                     trace!("Setting bar state to Err: {}", e);
-                    self.bar.set_message(&format!(
-                        "[{} {} {}]: State Err: {}",
-                        self.job_id, self.package_name, self.package_version, e
+                    self.bar.set_message(format!(
+                        "[{}/{} {} {} {}]: State Err: {}",
+                        self.endpoint_name, self.container_id_chrs, self.job_id, self.package_name, self.package_version, e
                     ));
                     success = Some(false);
                 }
@@ -376,19 +406,19 @@ impl<'a> LogReceiver<'a> {
         trace!("Finishing bar = {:?}", success);
         let finish_msg = match success {
             Some(true) => format!(
-                "[{} {} {}]: finished successfully",
-                self.job_id, self.package_name, self.package_version
+                "[{}/{} {} {} {}]: finished successfully",
+                self.endpoint_name, self.container_id_chrs, self.job_id, self.package_name, self.package_version
             ),
             Some(false) => format!(
-                "[{} {} {}]: finished with error",
-                self.job_id, self.package_name, self.package_version
+                "[{}/{} {} {} {}]: finished with error",
+                self.endpoint_name, self.container_id_chrs, self.job_id, self.package_name, self.package_version
             ),
             None => format!(
-                "[{} {} {}]: finished",
-                self.job_id, self.package_name, self.package_version
+                "[{}/{} {} {} {}]: finished",
+                self.endpoint_name, self.container_id_chrs, self.job_id, self.package_name, self.package_version
             ),
         };
-        self.bar.finish_with_message(&finish_msg);
+        self.bar.finish_with_message(finish_msg);
 
         if let Some(mut lf) = logfile {
             let _ = lf.flush().await?;
